@@ -13,6 +13,10 @@ from django.db import (
     models,
     transaction,
 )
+from django.db.models.fields.reverse_related import (
+    ManyToOneRel,
+    ManyToManyRel,
+)
 import graphene
 from graphene.relay.mutation import ClientIDMutation
 from graphene.types.mutation import MutationOptions
@@ -32,6 +36,7 @@ from .perms import (
     check_perms,
     check_authenticated,
 )
+from .settings import graphene_django_plus_settings
 from .utils import (
     get_node,
     get_nodes,
@@ -53,7 +58,10 @@ def _get_output_fields(model, return_field_name):
                 model.__name__,
             )
         )
-    f = graphene.Field(model_type, description="The mutated object.")
+    f = graphene.Field(
+        lambda: _registry.get_type_for_model(model),
+        description="The mutated object.",
+    )
     return {return_field_name: f}
 
 
@@ -83,6 +91,19 @@ def _get_fields(model, only_fields, exclude_fields, required_fields):
         for field in sorted(list(model._meta.fields + model._meta.many_to_many))
     ]
 
+    # Only add the field as input field if the foreign key (reverse side)
+    # can be set to null, otherwise updates won't work.
+    fields.extend(
+        [
+            (field.related_name or field.name + "_set", field)
+            for field in sorted(
+                list(model._meta.related_objects),
+                key=lambda field: field.name,
+            )
+            if not isinstance(field, ManyToOneRel) or field.remote_field.null
+        ],
+    )
+
     ret = collections.OrderedDict()
     for name, field in fields:
         if ((only_fields and name not in only_fields) or
@@ -98,29 +119,42 @@ def _get_fields(model, only_fields, exclude_fields, required_fields):
         elif isinstance(field, models.FileField):
             ret[name] = UploadType(
                 description=field.help_text,
-                required=not field.null,
             )
         elif isinstance(field, models.BooleanField):
             ret[name] = graphene.Boolean(
                 description=field.help_text,
-                required=not field.null and not field.blank,
             )
         elif isinstance(field, (models.ForeignKey, models.OneToOneField)):
             ret[name] = graphene.ID(
                 description=field.help_text,
-                required=not field.null,
             )
         elif isinstance(field, models.ManyToManyField):
             ret[name] = graphene.List(
                 graphene.ID,
                 description=field.help_text,
-                required=not field.null,
+            )
+        elif isinstance(field, (ManyToOneRel, ManyToManyRel)):
+            reverse_rel_include = graphene_django_plus_settings.MUTATIONS_INCLUDE_REVERSE_RELATIONS
+            # Checking whether it was globally configured to not include reverse relations
+            if isinstance(field, ManyToOneRel) and not reverse_rel_include and not only_fields:
+                continue
+
+            ret[name] = graphene.List(
+                graphene.ID,
+                description='Set list of {0}'.format(
+                    field.related_model._meta.verbose_name_plural,
+                ),
             )
         else:
             ret[name] = convert_django_field_with_choices(field, _registry)
 
         if required_fields is not None:
             ret[name].kwargs['required'] = name in required_fields
+        else:
+            if isinstance(field, (ManyToOneRel, ManyToManyRel)):
+                ret[name].kwargs['required'] = not field.null
+            else:
+                ret[name].kwargs['required'] = not field.blank
 
     return ret
 
@@ -350,6 +384,9 @@ class BaseModelMutation(BaseMutation):
         The easiest way when using `guardian` is to inherit it
         from :class:`graphene_django_plus.models.GuardedModel`.
         """
+        if not cls._meta.object_permissions:
+            return True
+
         if not isinstance(instance, GuardedModel):
             return True
 
@@ -501,7 +538,6 @@ class ModelMutation(BaseModelMutation):
         for f_name, f_item in cls.Input._meta.fields.items():
             if f_name not in data:
                 continue
-
             value = data[f_name]
 
             if value is not None and _is_list_of_ids(f_item):
@@ -543,15 +579,22 @@ class ModelMutation(BaseModelMutation):
         cls.clean_instance(instance, cleaned_input)
         cls.save(info, instance, cleaned_input)
 
-        # save m2m data
-        for f in itertools.chain(instance._meta.many_to_many,
-                                 instance._meta.private_fields):
-            if not hasattr(f, 'save_form_data'):
-                continue
-
-            d = cleaned_input.get(f.name, None)
-            if d is not None:
-                f.save_form_data(instance, d)
+        # save m2m and related object's data
+        for f in itertools.chain(
+            instance._meta.many_to_many,
+            instance._meta.related_objects,
+            instance._meta.private_fields,
+        ):
+            if isinstance(f, (ManyToOneRel, ManyToManyRel)):
+                # Handle reverse side relationships.
+                d = cleaned_input.get(f.related_name or f.name + "_set", None)
+                if d is not None:
+                    target_field = getattr(instance, f.related_name or f.name + "_set")
+                    target_field.set(d)
+            elif hasattr(f, 'save_form_data'):
+                d = cleaned_input.get(f.name, None)
+                if d is not None:
+                    f.save_form_data(instance, d)
 
         if not checked_permissions:
             # If we did not check permissions when getting the instance,
